clear all
close all
clc
%-------------------------------------------------%
% This a simulation model for a sailing hydrofoil vessel. There is a zero
% tolerance allowance for any foil to be entirely submerged, which stops
% terminates the simulation. For a further explenation see the
% Master Thesis 
% "An Unsteady Vortex Lattice Approach To Sailing Hydrofoil Dynamics."

% Specificed number of elements in chord and span directions respectively.
n = 10; 
m = 10;

dt = 0.1;       % Simulation time step
time = 20;      % Simulation time
t = dt:dt:time; % Simulation time vector
% Simulation requires a few time steps to develope the wake field for the
% foils and sail.
simstart = 30; 

% True wind angle, can either be a vector or single unit. Positive value
% indicates the wind is on the port side of the vessel.
wa = 25:5:75; 

% Truw Wind Speed, can either be a vector or single unit, negative value
% indicates upwind conditions, negative downwind. N.b the model is not
% accurate for downwind conditions.
Vt = -6;

% Initial value for saving results
pp = 0;

for ll = 1:length(Vt)
    
for jj = 1:length(wa)

    pp = pp+1;
    
    % Vessel Parameters are specificed in a different file
[wind,sail,hull,pos,foil] = simparam(n,m);
    
wind.angle = wa(jj);
wind.Vt = Vt(ll);
% Placeholder due to programming.
hydro.placeholder = 1;
aero.placeholder = 1;

%----Initial Conditions------%

% Creating initial conditions to be evaluated against the criteria
x = pos.crit(1,:); y = pos.crit(2,:); z = pos.crit(3,:);
cr = [x' y' z']; % Creating Initial Criteria
% Initial conditions in terms of angle and forward speed
Para.Vs = 7; Para.phi = 0; Para.theta = 0; Para.xi = 0; Para.h = 0;
Para.Vsy = 2;


% Initial conditions for vessel velocities
% p,q,r are the rotational velocities in roll, pitch and yaw respectively.
p = 0; q = 0; r = 0;
% u,v,w are the translational velocities in heave, surge and sway
% respectively.
u = Para.Vs; v = 0; w = 0;
% Initial values for time step and saving.
k = 0;
kk = 0;
% Critiera of the simulation
while (k < time/dt && all([cr(2,3) cr(4,3) cr(8,3)] < 0) && all([cr(1,3) cr(3,3) cr(7,3)] > 0))
% time step counter
k = k+1;
% Determining the rotational matrix
[B] = transmatrix(Para.phi,Para.theta,Para.xi);

Vel = [u; v; w];
% Translating the velocities from space to body fixed system.
Velw = B*Vel;
% Determining true wind angle and speed
[wind] = VelocityTriangle(Vel,wind,B);
 
sail.vel = -wind.Va; sail.dt = dt; sail.k = k;
% Saving the vessel orientation
sail.rot = B;
hydro.rot = B;
aero.rot = B;

[foil,amr,pos,hull] = geomdef(sail,wind,B,hull,pos,foil,Velw);

sail.pos = [0 0 1];
sail.p = Para.phi;

% Determining the lift and drag from the sail.
aero = liftsail(sail,wind,dt,k,aero,pos,simstart);

% Determining the lift and induced drag from each hydrofoil.
hydro = liftmodel(foil,Vel,dt,k,hydro,amr,pos,simstart);

% Translating the forces into the body fixed system
[Force] = transforce(B,hydro,aero,wind,hull);

if k == 1
[Ig,CG] = MMI(pos);
end
% Determining the added mass of the hydrofoil system.
hull.Ma = addedmass(hydro);

CG = [0 0 hull.char(5)];
% velocity vector for the equation of motion.
etadot = [u; v; w; p; q; r];

if k == 1
    
   momentarm = pos.momentarm; 
end

% Determining the total torque and force in each direciton.
[equil,hull] = equi(Force,momentarm,hull,Para);
% Solving Equation of Motion with respect to accelerations
dydt = solveEOM(equil,Ig,CG,hull,etadot);

% Euler time integration for translational velocities
   uvw = Vel + dydt(1:3)*dt*0.5;
    % Euler time integration for rotational velocities
    pqr = [p; q; r] + dydt(4:6)*dt;

if k > simstart
    
    p = pqr(1); q = pqr(2); r = pqr(3);
    u = uvw(1); v = uvw(2); w = uvw(3);
    % if applicable, remove comments from below to simulate less degreese
    % of freedom.
    u = Para.Vs;
    v = 0;
    w = 0;
    q = 0;
    r = 0;
%     p = 0;
end
Para.phi = Para.phi+p*dt;
Para.theta = Para.theta+q*dt;
Para.xi = Para.xi+r*dt;

% Saving current parameters
cr = pos.crit';
results.equil = equil;
results.uvw = Velw;
results.pqr = pqr;
results.hydro = hydro;
results.aero = aero;
results.ang = [Para.phi; Para.theta; Para.xi];
results.force = [Force.Ff Force.Fs Force.Fv];

subArea = 0;
for j = 1:4
    domain = hydro.domain(j);
    
    subArea = subArea+sum(sum(domain.S));
    clear domain
end

% Saving values for post-prcessing once the wake field is established.
if k > simstart
kk = kk+1;
ee.Sb(kk) = subArea;
ee.mx(kk) = equil.Mx;
ee.my(kk) = equil.My;
ee.mz(kk) = equil.Mz;
ee.fx(kk) = equil.Fx;
ee.fz(kk) = equil.Fz;
ee.v.x(kk) = u;
ee.v.y(kk) = v;
ee.v.z(kk) = w;
ee.v.p(kk) = p;
ee.v.q(kk) = q;
ee.v.r(kk) = r;
ee.ang.p(kk) = Para.phi;
ee.ang.q(kk) = Para.theta;
ee.ang.r(kk) = Para.xi;
ee.sp(kk) = sqrt(Velw(1)^2+Vel(2)^2+Vel(3)^2);
ee.lam(kk) = atand(Velw(2)/Vel(1));

ee.acc.x(kk) = dydt(1);
ee.acc.y(kk) = dydt(2);
ee.acc.z(kk) = dydt(3);
ee.acc.p(kk) = dydt(4);
ee.acc.q(kk) = dydt(5);
ee.acc.r(kk) = dydt(6);
end


end
% Displays time in simulation
display(['Finnished at ', num2str(k*dt)])
% Displays vessel orientaton
figure(124)
plot3(pos.foil.cord(:,1),pos.foil.cord(:,2),pos.foil.cord(:,3),'x')
hold on
plot3(pos.hull.cord(:,1),pos.hull.cord(:,2),pos.hull.cord(:,3),'^')
hold on
plot3(pos.sail.cord(:,1),pos.sail.cord(:,2),pos.sail.cord(:,3),'v')
hold off
axis equal
grid on
drawnow

% Saving results for post processing
saveresults(pp) = results;
saveresults2(pp) = ee;
% Resetting values for the next true wind angle/speed
clear hydro equil aero pos results B wind sail hull foil crit
clear aa Velw Vela u v w p q r Para AreA pqr uvw Vel Force amr angle
clear etadot ee kk k momentarm subArea Ig dydt x y z cr

end

end

